package com.ruoyi.invest.controller;

import java.util.List;
import java.io.IOException;
import javax.servlet.http.HttpServletResponse;
import javax.annotation.Resource;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.common.log.annotation.Log;
import com.ruoyi.common.log.enums.BusinessType;
import com.ruoyi.common.security.annotation.RequiresPermissions;
import com.ruoyi.invest.domain.InstallmentHistory;
import com.ruoyi.invest.domain.vo.InstallmentHistoryVo;
import com.ruoyi.invest.domain.dto.InstallmentHistoryDto;
import com.ruoyi.invest.service.IInstallmentHistoryService;
import com.ruoyi.common.core.web.controller.BaseController;
import com.ruoyi.common.core.web.domain.AjaxResult;
import com.ruoyi.common.core.utils.poi.ExcelUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import com.ruoyi.common.core.web.page.TableDataInfo;

/**
 * 网贷及分期历史Controller
 *
 * @author tianyongbao
 * @date 2024-03-20
 */
@Api(tags=" 网贷及分期历史")
@RestController
@RequestMapping("/installmentHistory")
public class InstallmentHistoryController extends BaseController
{
    @Resource
    private IInstallmentHistoryService installmentHistoryService;

    /**
     * 查询网贷及分期历史列表
     */
    @ApiOperation(value="查询网贷及分期历史列表",response = InstallmentHistoryVo.class)
    @RequiresPermissions("invest:installmentHistory:list")
    @GetMapping("/list")
    public TableDataInfo list(InstallmentHistoryDto installmentHistoryDto)
    {
        startPage();
        List<InstallmentHistoryVo> list = installmentHistoryService.selectInstallmentHistoryList(installmentHistoryDto);
        return getDataTable(list);
    }

    /**
     * 导出网贷及分期历史列表
     */
    @ApiOperation(value="导出网贷及分期历史列表")
    @RequiresPermissions("invest:installmentHistory:export")
    @Log(title = "网贷及分期历史", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, InstallmentHistoryDto installmentHistoryDto)
    {
        List<InstallmentHistoryVo> list = installmentHistoryService.selectInstallmentHistoryList(installmentHistoryDto);
        ExcelUtil<InstallmentHistoryVo> util = new ExcelUtil<InstallmentHistoryVo>(InstallmentHistoryVo.class);
        util.exportExcel(response, list, "网贷及分期历史数据");
    }

    /**
     * 获取网贷及分期历史详细信息
     */
    @ApiOperation(value="获取网贷及分期历史详细信息")
    @RequiresPermissions("invest:installmentHistory:query")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return success(installmentHistoryService.selectInstallmentHistoryById(id));
    }

    /**
     * 新增网贷及分期历史
     */
    @ApiOperation(value="新增网贷及分期历史")
    @RequiresPermissions("invest:installmentHistory:add")
    @Log(title = "网贷及分期历史", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody InstallmentHistory installmentHistory)
    {
        return toAjax(installmentHistoryService.insertInstallmentHistory(installmentHistory));
    }

    /**
     * 修改网贷及分期历史
     */
    @ApiOperation(value="修改网贷及分期历史")
    @RequiresPermissions("invest:installmentHistory:edit")
    @Log(title = "网贷及分期历史", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody InstallmentHistory installmentHistory)
    {
        return toAjax(installmentHistoryService.updateInstallmentHistory(installmentHistory));
    }

    /**
     * 删除网贷及分期历史
     */
    @ApiOperation(value="删除网贷及分期历史")
    @RequiresPermissions("invest:installmentHistory:remove")
    @Log(title = "网贷及分期历史", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids)
    {
        return toAjax(installmentHistoryService.deleteInstallmentHistoryByIds(ids));
    }
}
